# coding: utf-8
u"""Вспомогательные средства для работы с подсистемой RBAC."""
import operator

from m3.actions import Action
from m3.actions import ActionPack

from educommon.m3 import get_pack


def _resolve_packs_or_actions(packs_or_actions):
    u"""Возвращает список из паков и экшенов, заданных классами и строками."""
    def resolve(pack_or_action):
        if isinstance(pack_or_action, tuple):
            pack, action = pack_or_action
            pack = get_pack(pack)
            result = getattr(pack, action)
        else:
            result = get_pack(pack_or_action)
        return result

    result = [
        resolve(pack_or_action)
        for pack_or_action in packs_or_actions
    ]

    return result


def _rule_filter(rule_handler, packs_or_actions, operator_):
    u"""Возвращает обработчик правила, "обернутый" в фильтрующую функцию."""
    def action_in_list(action):
        for pack_or_action in _resolve_packs_or_actions(packs_or_actions):
            print pack_or_action
            if (
                isinstance(pack_or_action, Action) and
                action.__class__ is pack_or_action.__class__
            ) or (
                isinstance(pack_or_action, ActionPack) and
                action.parent is pack_or_action
            ):
                return True

        return False

    def wrapper(action, request, user):
        if operator_(action_in_list(action)):
            return rule_handler(action, request, user)
        else:
            return True

    return wrapper


def only_for(rule_handler, *packs_or_actions):
    u"""Выполняет правило только для указанных паков и экшенов.

    Экшены задаются в виде кортежа из двух элементов: первый элемент определяет
    пак (см. educommon.m3.get_pack()), а второй - имя атрибута этого пака, в
    котором содержится экшен. Иначе аргумент будет определять пак целиком.

    :param rule_handler: Обработчик правила.

    :rtype: callable
    """
    return _rule_filter(rule_handler, packs_or_actions, operator.truth)


def except_for(rule_handler, *packs_or_actions):
    u"""Выполняет правило для всех паков и экшенов, кроме указанных.

    Экшены задаются в виде кортежа из двух элементов: первый элемент определяет
    пак (см. educommon.m3.get_pack()), а второй - имя атрибута этого пака, в
    котором содержится экшен. Иначе аргумент будет определять пак целиком.

    :param rule_handler: Обработчик правила.

    :rtype: callable
    """
    return _rule_filter(rule_handler, packs_or_actions, operator.not_)
